// Progress.cpp : implementation file
//

#include "stdafx.h"
#include "Progress.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


/* Written by Amir Israeli January 2000,
   You may use this class , modify it , but dont remove this header.
   israelaq@wall.co.il  www.amir-israeli.tripod.com 
 Drawing stuff are taken from MSJ 1997 : A program of "Paul DiLascia"
 called GradientCap  (class name : CCaptionPainter) */

/////////////////////////////////////////////////////////////////////////////
// CProgress class

int CProgress::NCOLORSHADES = 64; // number of shades 
int CProgress::MOVEMENT = 3;
int CProgress::HLSMAX = 240; //240 This is what Display Properties uses
int CProgress::RGBMAX = 255; // max r/g/b value is 255
int CProgress::LUMINOSITY_MARK = 120;

CProgress::CProgress()
{
	rcBar = CRect(0,0,0,0);
	clrBK = clrFG = 0;
	m_nWidth=0;
	m_nTimerID=0;
	m_nPosition = 0;
	m_nDirection= UP;
	m_nFrequency = 50;
	m_bUseFadeEffect=m_bAllowResize=FALSE;
	m_bPlaying=FALSE;
	m_bShowText=FALSE;// show text or not
	m_nProgressPosition=0;
}

CProgress::~CProgress()
{
}


BEGIN_MESSAGE_MAP(CProgress, CWnd)
	//{{AFX_MSG_MAP(CProgress)
	ON_WM_CREATE()
	ON_WM_DESTROY()
	ON_WM_TIMER()
	ON_WM_PAINT()
	ON_WM_SIZE()
	//}}AFX_MSG_MAP
	ON_MESSAGE(WM_SETTEXT, OnSetText)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CProgress members and message handlers
BOOL CProgress::PreCreateWindow(CREATESTRUCT& cs) 
{
	static LPCTSTR lpszWndClass = AfxRegisterWndClass(CS_HREDRAW|CS_VREDRAW);
	cs.lpszClass = lpszWndClass;
	cs.style = WS_CHILDWINDOW|WS_VISIBLE;
	return CWnd::PreCreateWindow(cs);
}
BOOL CProgress::Create(UINT nID,CWnd *pParent,RECT &rc,BOOL bAllowResize,COLORREF clrBK)
{
	m_bAllowResize=bAllowResize;
	this->clrBK = clrBK;
	DWORD dwFlags = WS_CHILDWINDOW|WS_VISIBLE|WS_OVERLAPPED;
	return CWnd::Create(NULL,NULL,dwFlags,rc,pParent,nID,NULL);
}
BOOL CProgress::CreateFromStatic(UINT nID,CWnd *pParent,BOOL bAllowResize,COLORREF clrBK)
{
	m_bAllowResize=bAllowResize;
	this->clrBK = clrBK;
	CStatic wndStatic;
	if (!wndStatic.SubclassDlgItem(nID, pParent))
		return FALSE;

	// Get static control rect, convert to parent's client coords.
	CRect rc;
	wndStatic.GetWindowRect(&rc);
	pParent->ScreenToClient(&rc);
	wndStatic.DestroyWindow();
	return Create(nID,pParent,rc);
}

int CProgress::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CWnd::OnCreate(lpCreateStruct) == -1)
		return -1;
	// (A.I. : just a nice style I like)
	ModifyStyleEx(0,WS_EX_STATICEDGE,SWP_DRAWFRAME);
	CRect rc;
	GetClientRect(&rc);
	// default bar occupies the whole client area
	SetBar(rc,NCOLORSHADES);

	SetColor(CLR5,TRUE);
	return 0;
}
void CProgress::OnDestroy() 
{
	Pause();	
	CWnd::OnDestroy();
}

// sets scrolled bar and width of scoller area
void CProgress::SetBar(RECT &rc,int nWidth)
{
	rcBar = rc;
	m_nWidth = nWidth;
}
void CProgress::OnTimer(UINT nIDEvent) 
{
	if (m_nTimerID==nIDEvent) {
		Step();
		return;
	}
	CWnd::OnTimer(nIDEvent);
}

void CProgress::OnPaint() 
{
	CPaintDC dc(this); // device context for painting

	if (m_bPlaying) {
		PaintBar(dc,NULL);
	}
	else { // if not playing
		CRect rc;
		dc.GetClipBox(&rc);
		dc.FillSolidRect(&rc,clrBK);
	}
	// still needs some optimization
	DrawMyText(dc);
}

/////////////////////////////////////////////////////////////////////////////
// Play : create timer if not already created
BOOL CProgress::Play()
{
	if (m_nTimerID) return TRUE; //already got timer
	m_nTimerID = SetTimer(1000, m_nFrequency,NULL);
	m_bPlaying = TRUE;
	return (m_nTimerID!=0);
}
BOOL CProgress::Pause() 
{
	if (m_nTimerID!=0 && KillTimer(m_nTimerID)) {
		m_nTimerID=0;
		m_bPlaying = FALSE;
		m_nPosition = rcBar.left;
		m_nDirection= UP;
		Invalidate(NULL);
	}
	return (m_nTimerID==0);
}
int CProgress::SetFrequency(int nFrequency) 
{
	if (nFrequency<=20 && nFrequency>40000)
		return m_nFrequency; // invalid value
	if (m_nFrequency==nFrequency) 
		return m_nFrequency; // already was set
	m_nFrequency = nFrequency;
	
	// reset timer and start a new timer (with new frequency)
	if (m_bPlaying) {
		Pause();
		Play();
	}
	return m_nFrequency;
}
int CProgress::SetDirection(int nDirection)
{
	if (nDirection==UP || nDirection==DOWN)
		m_nDirection = nDirection;
	return GetDirection();
}
COLORREF CProgress::SetColor(COLORREF clr,BOOL bAddBrightness)
{
	DWORD dwMask=0xFF000000;
	clrFG = clr;
	if (bAddBrightness) clrFG |= dwMask;
	if (m_brush.GetSafeHandle())
		m_brush.DeleteObject();
	return (m_brush.CreateSolidBrush(clr)) ? clr : 0;
}

/* Several functions that are doing some text stuff :
 returns TRUE if text will be shown (FALSE if not) : a proposed 
 format is "Done %d\%" this will also set m_bShowText to TRUE */
BOOL CProgress::SetTextFormat(LPCTSTR lpsz)
{
	m_bShowText = (lpsz) ? TRUE : FALSE;
	m_nProgressPosition=0;	
	if (lpsz)
		strTextFormat = lpsz;
	return m_bShowText;
}
// sets position (percentage)
BOOL CProgress::SetProgressPosition(int nPos,BOOL bShowText)
{
	if (nPos>=0 && nPos<=100) {
		m_nProgressPosition=nPos;
		m_bShowText=bShowText;
		return TRUE;
	}
	return FALSE;
}
int CProgress::GetLuminosity(COLORREF clr)
{
	int r = GetRValue(clr);
	int g = GetGValue(clr);
	int b = GetBValue(clr);
	int rgbMax = max( max(r,g), b);
	int rgbMin = min( min(r,g), b);
	return (((rgbMax+rgbMin) * HLSMAX) + RGBMAX ) / (2*RGBMAX);
}
LRESULT CProgress::OnSetText(WPARAM wp, LPARAM lp)
{
	strTextFormat = (LPCTSTR)lp;
	if (strTextFormat.GetLength())
		Invalidate();
	return 0;	
}
void CProgress::DrawMyText(CDC &dc)
{
	if (!m_bShowText) return; // dont show text

	int nPrevMode = dc.SetBkMode(TRANSPARENT);	// draw on top of our shading
	if (GetLuminosity(clrFG) > LUMINOSITY_MARK)
		dc.SetTextColor(RGB(0,0,0));
	else dc.SetTextColor(RGB(255,255,255));

	CString strText;
	if (strTextFormat.GetLength())
		strText.Format(strTextFormat,m_nProgressPosition);
	else strText.Format("%d%c",m_nProgressPosition,37); //percentage character

	dc.DrawText(strText, &rcBar, DT_CENTER|DT_VCENTER|DT_SINGLELINE|DT_END_ELLIPSIS);
	dc.SetBkMode(nPrevMode);
}

// move 1 step to the next position : called from OnTimer
BOOL CProgress::Step()
{
	int iCurrentPosition = GetPosition();
	int iCurrentDirection = GetDirection();

	iCurrentPosition = (m_nDirection==UP) ? iCurrentPosition+MOVEMENT :
		iCurrentPosition-MOVEMENT;

	if (iCurrentPosition>=rcBar.right && m_nDirection==UP) {
		iCurrentPosition -= MOVEMENT;
		m_nDirection=DOWN;
	}
	else if (iCurrentPosition<rcBar.left && m_nDirection==DOWN) {
		iCurrentPosition += MOVEMENT;
		m_nDirection=UP;
	}
	// sets new position
	m_nPosition=iCurrentPosition;

	Invalidate();
	UpdateWindow();
	return TRUE;
}

// paint a small part of the bar : called many times
void CProgress::PaintBarArea(CDC &dc,int x,int y,int w,int h,COLORREF color)
{
	CBrush brush(color);
	CBrush* pOldBrush = dc.SelectObject(&brush);
	dc.PatBlt(x,y,w,h,PATCOPY);
	dc.SelectObject(pOldBrush);
}

/* Paints the bar of that is in progress :  whith some optimizations 
still needs some work - flickers when there's text within */
void CProgress::PaintBar(CDC &dc,LPCTSTR lpszTitle)
{
	COLORREF clrFrom = clrFG; // the beginning color ,
	COLORREF clrTo = clrBK;	// the ending color

	// Get the intensity values for the ending color
	int r1 = GetRValue(clrFrom); // red
	int g1 = GetGValue(clrFrom); // green
	int b1 = GetBValue(clrFrom); // blue
	
	// Get the intensity values for the begining color
	int r2 = GetRValue(clrTo); // red
	int g2 = GetGValue(clrTo); // green
	int b2 = GetBValue(clrTo); // blue

	static int x1,x2;
	if (m_nDirection==UP) {
		x1 = max(rcBar.left,m_nPosition-m_nWidth);
		x2 = m_nPosition;
	} else { // DOWN
		x1 = m_nPosition;
		x2 = min(rcBar.right,m_nPosition+m_nWidth);
	}

	/* static variables are instantiated once so quicker this way (and less 
	fragmentation : since this function is activated many times ) */
	static int xDelta= max(m_nWidth/NCOLORSHADES,1);	// width of one shade band
	static int r, g, b;
	static int pos,rDiff,gDiff,bDiff;

	rDiff = r1-r2; // caculate once for each (loop uses many times)
	gDiff = g1-g2;
	bDiff = b1-b2;
	if (m_nDirection==UP) {
		for (int i=x2 ; i>x1 ; i-=xDelta ) {
			pos= (x2-i);
			r = (r1>r2) ? (r1 - rDiff*pos/m_nWidth) : (r1 - rDiff*pos/m_nWidth);
			g = (g1>g2) ? (g1 - gDiff*pos/m_nWidth) : (g1 - gDiff*pos/m_nWidth);
			b = (b1>b2) ? (b1 - bDiff*pos/m_nWidth) : (b1 - bDiff*pos/m_nWidth);
			PaintBarArea(dc, i, rcBar.top, xDelta, rcBar.bottom, RGB(r, g, b));
		}
	} else {
		for (int i=x1 ; i<x2 ; i+=xDelta ) {
			pos= (i-x1);
			r = (r1>r2) ? (r1 - rDiff*pos/m_nWidth) : (r1 - rDiff*pos/m_nWidth);
			g = (g1>g2) ? (g1 - gDiff*pos/m_nWidth) : (g1 - gDiff*pos/m_nWidth);
			b = (b1>b2) ? (b1 - bDiff*pos/m_nWidth) : (b1 - bDiff*pos/m_nWidth);
			PaintBarArea(dc, i, rcBar.top, xDelta, rcBar.bottom, RGB(r, g, b));
		}
	}

	// extra area to paint 
	if (m_nDirection==UP) { 
		// paint area is to the left to the bar
		if (m_nPosition-m_nWidth>rcBar.left) {
			int nStartPoint = max(rcBar.left,m_nPosition-m_nWidth-xDelta);
			if (m_nPosition-m_nWidth-xDelta-MOVEMENT<rcBar.left)
				nStartPoint--;
			PaintBarArea(dc, nStartPoint, rcBar.top, xDelta, rcBar.bottom, clrTo);//RGB(r, g, b));
		}
	} else { //DOWN
		// paint area is to the right of bar
		if (m_nPosition+m_nWidth<rcBar.right) {
			int toPaint = min(xDelta,rcBar.right-(m_nPosition+m_nWidth));
			PaintBarArea(dc, m_nPosition+m_nWidth, rcBar.top, toPaint, rcBar.bottom, clrTo);//RGB(r, g, b));
		}
	}

	static int PIXELSALWAYSARE = 2; // pixels that will stay in target color
	// when bar reaches the end : (stay in dark color)
	if (!m_bUseFadeEffect) {
		if (m_nPosition+MOVEMENT>=rcBar.right) { // UP
			int toPaint = min(m_nWidth-PIXELSALWAYSARE,rcBar.right-m_nPosition-PIXELSALWAYSARE);
			PaintBarArea(dc, m_nPosition-m_nWidth, rcBar.top, m_nWidth-PIXELSALWAYSARE, rcBar.bottom, clrTo);

//			PaintBarArea(dc, m_nPosition-m_nWidth, rcBar.top, rcBar.right-(m_nPosition-m_nWidth)-PIXELSALWAYSARE, rcBar.bottom, clrTo);
		}
		else if (m_nPosition-MOVEMENT<rcBar.left) // DOWN
			PaintBarArea(dc, PIXELSALWAYSARE, rcBar.top, m_nPosition+m_nWidth, rcBar.bottom, clrTo);
	}

}

void CProgress::OnSize(UINT nType, int cx, int cy) 
{
	CWnd::OnSize(nType, cx, cy);
	if (m_bAllowResize)
		GetClientRect(&rcBar);
}
